@@ -498,7 +498,7 @@ egg_icon_list_init (EggIconList *icon_list)
   icon_list->priv->press_start_x = -1;
   icon_list->priv->press_start_y = -1;
   icon_list->priv->layout = gtk_widget_create_pango_layout (GTK_WIDGET (icon_list), NULL);
-  pango_layout_set_wrap (icon_list->priv->layout, PANGO_WRAP_CHAR);
+  pango_layout_set_wrap (icon_list->priv->layout, PANGO_WRAP_WORD);
   
   egg_icon_list_set_adjustments (icon_list, NULL, NULL);
 }
