@@ -1,10 +1,11 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * gnome-printer-list.c
- * Copyright (C) 2003, Ximian, Inc.
+ * Copyright (C) 2003-2004, Ximian, Inc.
  *
  * Authors:
  *   Dave Camp <dave@ximian.com>
+ *   Jody Goldberg <jody@novell.com>
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU Library General Public
@@ -38,6 +39,7 @@
 
 #include <libgnomecups/gnome-cups-printer.h>
 #include <libgnomecups/gnome-cups-ui-util.h>
+#include <libgnomecups/gnome-cups-permission.h>
 
 struct _GnomePrinterListDetails {
 	EggIconListItem *add_item;
@@ -52,19 +54,37 @@ static void disconnect_printer (GnomePrinterList *list, GnomeCupsPrinter *printe
 G_DEFINE_TYPE (GnomePrinterList, gnome_printer_list, egg_icon_list_get_type ())
 
 static GdkPixbuf *
-load_icon (char const *icon_name)
+load_icon (GtkWidget *w, char const *icon_name)
 {
-	return gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-		icon_name, 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+	GError *err = NULL;
+	GdkPixbuf *res = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+		icon_name, 48, 0, NULL);
+
+	if (res == NULL) {
+		if (err != NULL) {
+			g_warning ("%s", err->message);
+			g_error_free (err);
+			err = NULL;
+		}
+		res = gtk_widget_render_icon (w, icon_name, GTK_ICON_SIZE_DIALOG, NULL);
+
+		if (err != NULL) {
+			g_warning ("%s", err->message);
+			g_error_free (err);
+			return NULL;
+		}
+	}
+
+	return res;
 }
 
 static void
 add_new_printer_icon (GnomePrinterList *list)
 {
 	EggIconListItem *item;
-	GdkPixbuf *pixbuf;
-
-	pixbuf = load_icon ("gnome-dev-printer-new");
+	GdkPixbuf *pixbuf = load_icon (GTK_WIDGET (list), "gnome-dev-printer-new");
+	if (pixbuf == NULL)
+		pixbuf = load_icon (GTK_WIDGET (list), GTK_STOCK_PRINT);
 
 	item = egg_icon_list_item_new (pixbuf, _("New Printer"));
 	list->details->add_item = item;
@@ -75,16 +95,42 @@ add_new_printer_icon (GnomePrinterList *list)
 }
 
 static GdkPixbuf *
-get_printer_icon (GnomeCupsPrinter *printer)
+get_printer_icon (GtkWidget *list, GnomeCupsPrinter *printer)
 {
 	char *icon_name;
-	GdkPixbuf *pixbuf;
+	GList *ptr, *emblems;
+	GdkPixbuf *pixbuf, *emblem;
+	GError *err = NULL;
 
 	gnome_cups_printer_get_icon (printer,
 				     &icon_name,
-				     NULL);
-	pixbuf = load_icon (icon_name);
+				     &emblems);
+	pixbuf = load_icon (GTK_WIDGET (list), icon_name);
 	g_free (icon_name);
+	if (pixbuf == NULL)
+		pixbuf = load_icon (GTK_WIDGET (list), GTK_STOCK_PRINT);
+
+	for (ptr = emblems; ptr != NULL; ptr = ptr->next) {
+		if (ptr->data == NULL)
+			continue;
+		emblem = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+			ptr->data, 48, 0, &err);
+		if (emblem == NULL) {
+			g_warning ("unable to load emblem '%s'", (char *)ptr->data);
+			if (err != NULL) {
+				g_warning ("%s", err->message);
+				g_error_free (err);
+				err = NULL;
+			}
+		} else {
+			GdkPixbuf *res = gdk_pixbuf_copy (pixbuf);
+			gdk_pixbuf_composite (emblem,  res, 0, 0, 
+				gdk_pixbuf_get_width (res), gdk_pixbuf_get_height (res), 
+				0., 0., 1., 1., GDK_INTERP_NEAREST, 255);
+			g_object_unref (pixbuf);
+			pixbuf = res;
+		}
+	}
 
 	return pixbuf;
 }
@@ -92,25 +138,28 @@ get_printer_icon (GnomeCupsPrinter *printer)
 static char *
 get_printer_label (GnomeCupsPrinter *printer)
 {
-	char *label;
-	
-	/* FIXME: hard-coded additional text color */
-	label = g_strdup_printf ("%s\n<span color=\"#33337F\">%s\n%d Jobs</span>",
-				 gnome_cups_printer_get_name (printer),
-				 gnome_cups_printer_get_state_name (printer),
-				 gnome_cups_printer_get_job_count (printer));
+	char const *name  = gnome_cups_printer_get_name (printer);
+	char const *state = gnome_cups_printer_get_state_name (printer);
+	int jobs = gnome_cups_printer_get_job_count (printer);
 
-	return label;
+	/* FIXME: hard-coded additional text color */
+	if (jobs > 0) {
+		return g_strdup_printf ("%s\n<span color=\"#337F33\">%s\n%d Jobs</span>",
+			name, state, jobs);
+	} else {
+		return g_strdup_printf ("%s\n<span color=\"#33337F\">%s\n</span>",
+			name, state);
+	}
 }
 
 static EggIconListItem *
-create_printer_item (GnomeCupsPrinter *printer)
+create_printer_item (GtkWidget *list, GnomeCupsPrinter *printer)
 {
 	EggIconListItem *item;
 	GdkPixbuf *pixbuf;
 	char *label;
 	
-	pixbuf = get_printer_icon (printer);
+	pixbuf = get_printer_icon (list, printer);
 	label = get_printer_label (printer);
 	
 	item = egg_icon_list_item_new (pixbuf, label);
@@ -180,12 +229,6 @@ properties_activated_cb (GtkWidget *menu_item,
 	gnome_printer_list_selection_properties (GNOME_PRINTER_LIST (user_data));
 }
 
-static gboolean
-need_admin_priviledges (void)
-{
-	return (geteuid () != 0);
-}
-
 /* This is so lame */
 void
 gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
@@ -193,8 +236,9 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 					      gboolean *pause,
 					      gboolean *resume,
 					      gboolean *make_default,
-					      gboolean *delete,
-					      gboolean *properties)
+					      gboolean *del,
+					      gboolean *properties,
+					      gboolean *status)
 {
 	GList *l;
 	GList *selection;
@@ -222,12 +266,15 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 		if (make_default) {
 			*make_default = FALSE;
 		}
-		if (delete) {
-			*delete = FALSE;
+		if (del) {
+			*del = FALSE;
 		}
 		if (properties) {
 			*properties = FALSE;
 		}
+		if (status) {
+			*status = FALSE;
+		}
 		return;
 	}
 	
@@ -264,25 +311,29 @@ gnome_printer_list_get_command_sensitivities (GnomePrinterList *list,
 	}
 	
 	if (pause) {
-		*pause = !new_printer_selected && !all_paused;
+		*pause = !new_printer_selected && !all_paused && gnome_cups_can_admin ();
 	}
 	
 	if (resume) {
-		*resume = !new_printer_selected && !all_running;
+		*resume = !new_printer_selected && !all_running && gnome_cups_can_admin ();
 	}
 	
 	if (make_default) {
 		*make_default = !new_printer_selected && num_selected == 1 && !has_default;
 	}
 	
-	if (delete) {
-		*delete = !new_printer_selected && ! need_admin_priviledges ();
+	if (del) {
+		*del = !new_printer_selected && gnome_cups_can_admin ();
 	}
 	
 	if (properties) {
 		*properties = !new_printer_selected;
 	}
 
+	if (status) {
+		*status = !new_printer_selected;
+	}
+
 	g_list_free (selection);
 }
 
@@ -296,35 +347,40 @@ create_selection_menu (GnomePrinterList *list)
 	gboolean pause;
 	gboolean resume;
 	gboolean make_default;
-	gboolean delete;
+	gboolean del;
 	gboolean properties;
+	gboolean status;
 
 	gnome_printer_list_get_command_sensitivities (list,
-						      &activate,
-						      &pause,
-						      &resume,
-						      &make_default,
-						      &delete,
-						      &properties);
+		&activate, &pause, &resume, &make_default,
+		&del, &properties, &status);
 	
 	selection = egg_icon_list_get_selected (EGG_ICON_LIST (list));
 
 	menu = gtk_menu_new ();
 	gtk_widget_show (menu);
 
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_OPEN, NULL);
+	if (g_list_length (selection) == 1 && selection->data == list->details->add_item) {
+		g_list_free (selection);
+
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD, NULL);
+		gtk_widget_show (item);
+		g_signal_connect (item, "activate", 
+				  G_CALLBACK (open_activated_cb),
+				  list);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		return menu;
+	}
+	g_list_free (selection);
+
+	item = gtk_menu_item_new_with_label (_("Jobs"));
 	gtk_widget_show (item);
 	g_signal_connect (item, "activate", 
 			  G_CALLBACK (open_activated_cb),
 			  list);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	if (g_list_length (selection) == 1 && selection->data == list->details->add_item) {
-		g_list_free (selection);
-		return menu;
-	}
-	g_list_free (selection);
-
 	item = gtk_separator_menu_item_new ();
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
@@ -357,9 +413,9 @@ create_selection_menu (GnomePrinterList *list)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_menu_item_new_with_label (_("Delete"));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, NULL);
 	gtk_widget_show (item);
-	gtk_widget_set_sensitive (item, delete);
+	gtk_widget_set_sensitive (item, del);
 	g_signal_connect (item, "activate", 
 			  G_CALLBACK (delete_activated_cb),
 			  list);
@@ -369,7 +425,7 @@ create_selection_menu (GnomePrinterList *list)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);	
 
-	item = gtk_menu_item_new_with_label (_("Properties"));
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
 	gtk_widget_show (item);
 	gtk_widget_set_sensitive (item, properties);
 	g_signal_connect (item, "activate", 
@@ -431,7 +487,7 @@ printer_changed_cb (GnomeCupsPrinter *printer,
 	list = GNOME_PRINTER_LIST (user_data);
 	item = g_object_get_data (G_OBJECT (printer), "icon_item");
 	
-	pixbuf = get_printer_icon (printer);
+	pixbuf = get_printer_icon (GTK_WIDGET (list), printer);
 	label = get_printer_label (printer);
 	
 	egg_icon_list_item_set_icon (item, pixbuf);
@@ -489,7 +545,7 @@ watch_printer (GnomePrinterList *list,
 						 printer);
 	g_object_ref (printer);
 	
-	item = create_printer_item (printer);
+	item = create_printer_item (GTK_WIDGET (list), printer);
 	g_object_set_data (G_OBJECT (printer), "icon_item", item);
 
 	egg_icon_list_append_item (EGG_ICON_LIST (list), item);
@@ -544,24 +600,11 @@ icon_compare (EggIconList *icon_list,
 		       gnome_cups_printer_get_name (printer_b));
 }
 
-extern char *gnome_cups_manager_execname (char const *name);
 void
 gnome_printer_list_new_printer (GnomePrinterList *list)
 {
-        char *argv[] = { NULL, NULL };
-        GError *error;
-        
-        error = NULL;
-
-	argv[0] = gnome_cups_manager_execname ("gnome-cups-add");
-        g_spawn_async (NULL, argv, NULL, 
-                       G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
-
-        if (error) {
-		gnome_cups_error_dialog (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (list))), NULL, error);
-		g_error_free (error);
-        }
-        g_free (argv [0]);
+	gnome_cups_spawn ("gnome-cups-add", 0, NULL,
+		TRUE, gtk_widget_get_toplevel (GTK_WIDGET (list)));
 }
 
 void
