@@ -617,7 +617,7 @@ setup_driver_page (GladeXML *xml, GnomeCupsPrinter *printer)
 
 		g_signal_connect_object (ds,
 			"changed",
-			G_CALLBACK (cb_driver_changed), xml, 0);
+			G_CALLBACK (cb_driver_changed), printer, 0);
 	}
 	ppdClose (ppd);
 }
@@ -625,29 +625,50 @@ setup_driver_page (GladeXML *xml, GnomeCupsPrinter *printer)
 static void
 cb_connection_changed (GCupsConnectionSelector *cs, GnomeCupsPrinter *printer)
 {
+	char *uri = gcups_connection_selector_get_uri (cs);
+
+	if (uri != NULL) {
+		ipp_t *request = gnome_cups_request_new_for_printer (
+			CUPS_ADD_PRINTER, printer);
+		ippAddString (request, IPP_TAG_PRINTER, IPP_TAG_NAME,
+			"device-uri", NULL, gnome_cups_strdup (uri));
+		gnome_cups_request_execute_async (request, NULL, "/admin/", NULL, NULL, NULL);
+		g_free (uri);
+	}
 }
+
+static void
+cb_display_uri (GnomeCupsPrinter *printer, GCupsConnectionSelector *cs)
+{
+	char const *uri = gnome_cups_printer_get_device_uri (printer);
+	if (uri == NULL)
+		return;
+	gcups_connection_selector_set_uri (cs, uri);
+}
+
 static void
 setup_connection_page (GladeXML *xml, GnomeCupsPrinter *printer)
 {
 	GtkWidget  *w, *cs;
-	char const *uri;
-
 	if (!gnome_cups_can_admin ())
 		return;
 	if (printer == NULL)
 		return;
-	uri = gnome_cups_printer_get_device_uri (printer);
-	if (uri == NULL)
-		return;
 
 	cs = gcups_connection_selector_new ();
 	w = glade_xml_get_widget (xml, "prop_notebook");
 	gtk_notebook_append_page (GTK_NOTEBOOK (w),
 		cs, gtk_label_new (_("Connection")));
-	gcups_connection_selector_set_uri (GCUPS_CONNECTION_SELECTOR (cs),
-		uri);
+	if (gnome_cups_printer_get_attributes_initialized (printer))
+		cb_display_uri (printer, GCUPS_CONNECTION_SELECTOR (cs));
+	else if (NULL == g_object_get_qdata (G_OBJECT (cs), g_quark_from_static_string ("pending"))) {
+		gulong handle = g_signal_connect_object (printer, "attributes-changed",
+			G_CALLBACK (cb_display_uri), cs, 0);
+		g_object_set_qdata (G_OBJECT (cs),
+			g_quark_from_static_string ("pending"),
+			GUINT_TO_POINTER (handle));
+	}
 	gtk_widget_show (cs);
-
 	g_signal_connect_object (cs,
 		"changed",
 		G_CALLBACK (cb_connection_changed), xml, 0);
