@@ -25,8 +25,8 @@
 #include <libgnomecups/gnome-cups-printer.h>
 #include <libgnomecups/gnome-cups-queue.h>
 #include <libgnomecups/gnome-cups-ui-print.h>
+#include <libgnomecups/gnome-cups-ui-util.h>
 
-#include "util.h"
 #include "printer-properties.h"
 
 enum {
@@ -49,13 +49,7 @@ enum {
 static char*
 format_size_for_display (unsigned long size)
 {
-        if (size < (unsigned long)KILOBYTE_FACTOR) {
-                if (size == 1)
-                        return g_strdup (_("1 byte"));
-                else
-                        return g_strdup_printf (_("%u bytes"),
-                                                       (guint) size);
-        } else {
+        if (size >= (unsigned long)KILOBYTE_FACTOR) {
                 gdouble displayed_size;                                                      
                 if (size < (unsigned long) MEGABYTE_FACTOR) {
                         displayed_size = (gdouble) size / KILOBYTE_FACTOR;
@@ -70,7 +64,8 @@ format_size_for_display (unsigned long size)
                         return g_strdup_printf (_("%.1f GB"),
                                                        displayed_size);
                 }
-        }
+        } else
+		return g_strdup_printf (ngettext ("1 byte", "%lu bytes", size ), size);
 }
 
 static void
