@@ -42,6 +42,7 @@
 #include <gtk/gtkmessagedialog.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
+#include <glib/gstrfuncs.h>
 #include <stdio.h>
 #include <errno.h>
 
@@ -260,6 +261,7 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 {
 	char *model, *driver;
 	char *tmp, *without_vendor;
+        char *language, *language_position;
 
 	if (alias_to_vendor_map == NULL) {
 		static char const *apple[] = { "APPLE", NULL };
@@ -298,15 +300,25 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 	if (NULL == without_vendor)
 		without_vendor = (char *)ppd->nickname;
 
-	/* RH style entries
-	 * 	<model>, <driver> */
-	if (NULL != (tmp = strstr (without_vendor, ", "))) {
-		model = g_strndup (without_vendor, tmp - without_vendor);
-		driver = g_strdup (tmp + 2);
+	g_return_if_fail (without_vendor != NULL);
 
-	/* Stock foomatic entries (suse, debian)
+	/* gimp-print ijs models (suse, debian)
+	 * 	<model> Foomatic/gimp-print-ijs */
+	if (NULL != (tmp = strstr (without_vendor, " Foomatic/gimp-print-ijs"))) {
+		model = g_strndup (without_vendor, tmp - without_vendor);
+		driver = g_strdup (_("High Quality Image (GIMP-Print Inkjet)"));
+	}
+
+	/* gimp-print and ijs models (suse, debian)
+	 * 	<model> Foomatic/gimp-print */
+	else if (NULL != (tmp = strstr (without_vendor, " Foomatic/gimp-print"))) {
+		model = g_strndup (without_vendor, tmp - without_vendor);
+		driver = g_strdup (_("High Quality Image (GIMP-Print)"));
+	}
+
+        /* Stock foomatic entries (suse, debian)
 	 * 	<model> Foomatic/<driver> */
-	} else if (NULL != (tmp = strstr (without_vendor, " Foomatic/"))) {
+	else if (NULL != (tmp = strstr (without_vendor, " Foomatic/"))) {
 		model = g_strndup (without_vendor, tmp - without_vendor);
 		driver = g_strdup (tmp + 10);
 
@@ -317,12 +329,27 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 		driver = g_strdup (_("Standard (CUPS)"));
 
 	/* gimp-print builtin
-	 * 	<MODEL_IN_CAPS> - CUPS+Gimp-Print vm.n.o' */
+	 * 	<MODEL_IN_CAPS> - CUPS+Gimp-Print vm.n.o'
+         * The filename may have a subdirectory--treat the subdir as a language, and set
+         * the driver string accordingly. Use the regular driver string for the "C" language. */
 	} else if (NULL != (tmp = strstr (without_vendor, " - CUPS+Gimp-Print v"))) {
 		model = g_strndup (without_vendor, tmp - without_vendor);
-		driver = g_strdup (_("High Quality Image (GIMP-Print)"));
+                if((NULL != (language_position = strstr (ppd->filename, "/"))) &&
+                   strncmp ("C/", ppd->filename, 2) &&
+                   strncmp ("stp/", ppd->filename, 4)) {
+                    language = g_strndup (ppd->filename, language_position - ppd->filename);
+                    driver = g_strdup_printf (_("High Quality Image (GIMP-Print) (%s)"), language);
+                    g_free(language);
+                } else {
+                    driver = g_strdup (_("High Quality Image (GIMP-Print)"));
+                }
 
-	/* hmm we can not parse it in a known structure but at least we have an idea of the vendor */
+	/* RH style entries
+	 * 	<model>, <driver> */
+	} else if (NULL != (tmp = strstr (without_vendor, ", "))) {
+		model = g_strndup (without_vendor, tmp - without_vendor);
+		driver = g_strdup (tmp + 2);
+        /* hmm we can not parse it in a known structure but at least we have an idea of the vendor */
 	} else if (without_vendor != ppd->nickname) {
 		model = g_strdup (without_vendor);
 		driver = g_strdup (_("Standard"));
@@ -333,7 +360,7 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 	}
 
 	/* Lets see if this is a foomatic 'recommended' driver */
-	if (NULL != (tmp = strstr (driver, " (recommended)"))) {
+	if (NULL != (tmp = strstr (without_vendor, " (recommended)"))) {
 		strcpy (tmp, tmp+14);
 		ppd->is_recommended = TRUE;
 	} else
@@ -349,25 +376,31 @@ ppd_extract_model_and_driver (GCupsPPD *ppd)
 static int
 compare_models_by_driver (gconstpointer a, gconstpointer b)
 {
-	const GCupsPPD *a1 = a;
-	const GCupsPPD *b1 = b;
+	GCupsPPD const *a1 = a;
+	GCupsPPD const *b1 = b;
+	if (a1->is_recommended ^ b1->is_recommended)
+		return a1->is_recommended ? -1 : 1;
 	return str_case_compare (a1->driver, b1->driver);
 }
 
 static void
-categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
+setup_ppd (GCupsDriverSelector *ds, GCupsPPD *ppd, gboolean select)
 {
-	GSList *drivers = NULL, *dup;
-	GHashTable *models = g_hash_table_lookup (vendors, ppd->vendor);
+	GSList *drivers = NULL, *tmp;
+	GHashTable *models;
+
+	ppd_extract_model_and_driver (ppd);
+
+	models = g_hash_table_lookup (ds->vendors, ppd->vendor);
 	if (models == NULL) {
 		models = g_hash_table_new_full (
 			str_case_hash, str_case_equal, NULL,
 			(GDestroyNotify)ppd_list_free);
-		g_hash_table_insert (vendors, ppd->vendor, models);
-		if (xml != NULL) {
+		g_hash_table_insert (ds->vendors, ppd->vendor, models);
+		if (select) {
 			GtkTreeIter   iter;
 			GtkTreeModel *store = gtk_combo_box_get_model (
-				GTK_COMBO_BOX (glade_xml_get_widget (xml, "vendors")));
+				GTK_COMBO_BOX (glade_xml_get_widget (ds->xml, "vendors")));
 			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
 				0, ppd->vendor,
@@ -376,33 +409,50 @@ categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
 	} else
 		drivers = g_hash_table_lookup (models, ppd->model);
 
-	if (xml != NULL && drivers == NULL) {
+	if (select && drivers == NULL) {
 		GtkTreeIter   iter;
 		GtkTreeModel *store = gtk_tree_view_get_model (
-			GTK_TREE_VIEW (glade_xml_get_widget (xml, "model_treeview")));
+			GTK_TREE_VIEW (glade_xml_get_widget (ds->xml, "model_treeview")));
 		gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (store), &iter,
 			0, ppd->model,
 			-1);
 	}
 
-	if (NULL != (dup = g_slist_find_custom (drivers, ppd, compare_models_by_driver))
-	    && dup->data != ppd) {
-		GCupsPPD const *existing = dup->data;
-		g_warning ("Two ppds have driver == '%s'\n\t->%s (%s) and\n\t->%s (%s)",
+	if (NULL != (tmp = g_slist_find_custom (drivers, ppd, compare_models_by_driver))
+	    && tmp->data != ppd) {
+		GCupsPPD *existing = tmp->data;
+
+		/* SuSE likes to tweak some of the ppds and install both copies.
+		 * Give the tweaked versions perference.  Drop stp variants in
+		 * preference to others */
+		if (NULL != strstr (ppd->filename, "suse.ppd") ||
+		    NULL != strstr (existing->filename, "stp/")) {
+			char *tmp = existing->filename;
+			existing->filename = ppd->filename;
+			ppd->filename = tmp;
+			ppd_free (ppd); /* prefer existing, drop the new one */
+			return;
+		}
+		if (NULL != strstr (existing->filename, "suse.ppd") ||
+		    NULL != strstr (ppd->filename, "stp/")) {
+			ppd_free (ppd); /* prefer existing, drop the new one */
+			return;
+		}
+		g_warning ("Two ppds have driver == '%s'\n\t->%s (%s[%d]) and\n\t->%s (%s)[%d]",
 			   ppd->driver,
-			   ppd->filename,	ppd->nickname,
-			   existing->filename,	existing->nickname);
+			   ppd->filename,	ppd->nickname, ppd->is_recommended,
+			   existing->filename,	existing->nickname, existing->is_recommended);
 	}
 
 	drivers = g_slist_insert_sorted (drivers, ppd, compare_models_by_driver);
 	g_hash_table_steal (models, ppd->model);
 	g_hash_table_insert (models, ppd->model, drivers);
 
-	if (xml != NULL) {
+	if (select) {
 		GdkPixbuf    *pixbuf;
 		GtkTreeIter   iter;
-		GtkComboBox  *combo = GTK_COMBO_BOX (glade_xml_get_widget (xml, "drivers"));
+		GtkComboBox  *combo = GTK_COMBO_BOX (glade_xml_get_widget (ds->xml, "drivers"));
 		GtkTreeModel *store = gtk_combo_box_get_model (combo);
 		gtk_list_store_append (GTK_LIST_STORE (store), &iter);
 		if (ppd->is_recommended)
@@ -417,21 +467,14 @@ categorize_ppd (GCupsPPD *ppd, GHashTable *vendors, GladeXML *xml)
 			3, pixbuf,
 			-1);
 	}
-}
-
-static void
-setup_ppd (GCupsDriverSelector *ds, GCupsPPD *ppd, gboolean select)
-{
-	ppd_extract_model_and_driver (ppd);
-	categorize_ppd (ppd, ds->vendors, select ? ds->xml : NULL);
 	g_hash_table_insert (ds->ppds, ppd->nickname, ppd);
 }
 
 static GCupsPPD *
-add_ppd (GCupsDriverSelector *ds, char *name, char *vendor, char *nickname)
+add_ppd (GCupsDriverSelector *ds, char *filename, char *vendor, char *nickname)
 {
 	GCupsPPD *ppd = ppd_new ();
-	ppd->filename = name;
+	ppd->filename = filename;
 	ppd->vendor = vendor;
 	ppd->nickname = nickname;
 	setup_ppd (ds, ppd, TRUE);
@@ -710,19 +753,27 @@ static void
 cb_driver_changed (GCupsDriverSelector *ds)
 {
 	/* some debug spew */
+#if 1
 	GCupsPPD const *ppd = gcups_driver_selector_get (ds);
 	if (ppd != NULL)
 		g_print ("Selected ppd file = %s\n", ppd->filename);
+#endif
 
 	g_signal_emit (G_OBJECT (ds), signals [CHANGED], 0);
 }
 
 #define CUPS_DIR  "/usr/share/cups/model"
 
+static gboolean
+cb_find_ppd_by_filename (gpointer key, gpointer value, gpointer filename)
+{
+	return (0 == strcmp (filename, ((GCupsPPD const *)value)->filename));
+}
+
 static void
 cb_install_driver (GCupsDriverSelector *ds)
 {
-	GCupsPPD   *gppd;
+	GCupsPPD   *gppd = NULL;
 	ppd_attr_t *manufacturer, *nickname;
 	ppd_file_t *ppd = NULL;
 	FILE *stdio = NULL;
@@ -789,6 +840,7 @@ cb_install_driver (GCupsDriverSelector *ds)
 	if (g_file_test (target_path, G_FILE_TEST_EXISTS)) {
 		err = g_error_new (1, 1,
 			_("The PPD\n\t<b>%s</b>\nis already installed"), target_path);
+		gppd = g_hash_table_find (ds->ppds, &cb_find_ppd_by_filename, base);
 		goto done;
 	}
 
@@ -820,11 +872,13 @@ cb_install_driver (GCupsDriverSelector *ds)
 	gppd = add_ppd (ds, base,
 		 g_strdup (manufacturer ? manufacturer->value : "Raw Queue"),
 		 g_strdup (nickname ? nickname->value : "User supplied"));
-	combo_select (ds->xml, "vendors", 0, gppd->vendor);
-	select_model (ds->xml, gppd->model);
-	combo_select (ds->xml, "drivers", 0, gppd->driver);
-
 done :
+	if (gppd != NULL) {
+		combo_select (ds->xml, "vendors", 0, gppd->vendor);
+		select_model (ds->xml, gppd->model);
+		combo_select (ds->xml, "drivers", 0, gppd->driver);
+	}
+
 	if (ppd != NULL)
 		ppdClose (ppd);
 	if (stdio != NULL)
